=head1 NAME

Baby

=head1 DESCRIPTION

B<Baby> objects do not have any points of their own, but
contain point objects of polygon objects.  Babies can become
polygon objects.

=head1 AUTHOR

B<Alex Bateman> Email agb@sanger.ac.uk

=cut

#
# Perl Module for Baby
#
# Cared for by Alex Bateman <agb@sanger.ac.uk>
#

package Baby;

use vars qw($AUTOLOAD @ISA @EXPORT_OK);
use Exporter;
use Carp;
use strict;
use Point;
use Polygon;
use Line;

#
# Place functions/variables you want to *export*, ie be visible from the caller package into @EXPORT_OK
#

@EXPORT_OK = qw();

#
# @ISA has our inheritance.
#


@ISA = ( 'Exporter','Polygon');

#######
# new #
#######

=head1 new


Creates empty Baby polygon object
centre of scaling and rotation is passed as an argument that
is converted into a Point object.
This centre is also the vertex of the dual tiling!

$polygon = new Baby($x,$y);

=cut

sub new {
    my($class,$x,$y)=@_;

    my $polygon = new Polygon($x,$y);

    bless $polygon, $class;
    return($polygon);
}


#######
# add #
#######

=head1 add

Method to add line objects to a baby polygon object

$polygon->add($point,$point2...);

=cut

sub add {
  my $polygon = shift @_;
  while (@_){
    my $line = shift @_;
    push(@{$polygon->{'lines'}},$line);
  }
}

##########
# colour #
##########

=head1 colour

Dummy method. Baby objects cannot have a colour

$baby->colour();

=cut

sub colour {
#  print "Unreachable code finally executed! Cannot Scale baby!\n";
}

#########
# scale #
#########

=head1 scale

Dummy method. Baby objects cannot be scaled!

$baby->scale();

=cut

sub scale {
#  print "Unreachable code finally executed! Cannot Scale baby!\n";
  return 1;
}

##########
# rotate #
##########

=head1 rotate

Dummy method. Baby objects cannot be rotated!

$baby->rotate();

=cut

sub rotate {
#  warn "Unreachable code finally executed! Cannot rotate baby!\n";
  return 1;

}

#############
# translate #
#############

=head1 translate

Dummy method. Baby objects cannot be translated!

$baby->translate($x,$y);

=cut

sub translate {
  die "Cannot translate baby!\n";
}


##########
# render #
##########
=head1 render

Method to return an XML render stack, that will be used
by View objects.  Also this method returns an array
which has object references in.  These correspond to
tag attributes for objects in XML.  THis means external 
objects to model can refer to objects by these tags and
we can work out which object the external objects mean.

($stack,$tag_ref)=$tiling->render($current_tag,$stack,\@tags);

$stack is XML stack for rendering
$tag_ref is a reference to the array of objects

=cut
sub render {
    my ($self,$current_tag,$stack_ref,$tag_ref) = @_;

    $current_tag++;
    push (@$tag_ref,$self);

    my $open_tag  = "    <polygon tag=\"$current_tag\" type=\"baby\">\n";
    my $close_tag = "    <polygon/>\n";

    $$stack_ref .= $open_tag;
    foreach my $element ($self->eachline()){
	($current_tag,$stack_ref,$tag_ref)=$element->render($current_tag,$stack_ref,$tag_ref);
    }
    $$stack_ref .= $close_tag;

    return ($current_tag,$stack_ref,$tag_ref);
}

##############
# to_polygon #
##############

=head1 to_polygon
Creates new polygon object from baby object

    $polygon = to_polygon($baby);
=cut

sub to_polygon {
  my $baby = shift @_;

  my ($x,$y)=$baby->dual_vertex()->xy();

  # Make new polygon
  my $polygon = new Polygon($x,$y);

  foreach my $line ($baby->eachline()){
    # Copy point
    my $new_line = $line->copy();
    # Add line to new polygon
    $polygon->add($new_line);
  }
  $baby=undef; # Destroy old object
  return $polygon;
}

1;  # says use was ok



__END__

