=head1 NAME

Edge

=head1 DESCRIPTION

Description for B<Edge>

=head1 AUTHOR

B<Alex Bateman> Email agb@sanger.ac.uk

=cut

#
# Perl Module for Edge
#
# Cared for by Alex Bateman <agb@sanger.ac.uk>
#

package Edge;

use vars qw($AUTOLOAD @ISA @EXPORT_OK);
use Exporter;
use Carp;
use strict;
use Point;


#
# Place functions/variables you want to *export*, ie be visible from the caller package into @EXPORT_OK
#

@EXPORT_OK = qw();

#
# @ISA has our inheritance.
#

@ISA = ( 'Exporter','Baby');



##########
# render #
##########
=head1 render

Method to return an XML render stack, that will be used
by View objects.  Also this method returns an array
which has object references in.  These correspond to
tag attributes for objects in XML.  THis means external 
objects to model can refer to objects by these tags and
we can work out which object the external objects mean.

($stack,$tag_ref)=$tiling->render($current_tag,$stack,\@tags);

$stack is XML stack for rendering
$tag_ref is a reference to the array of objects

=cut
sub render {
    my ($self,$current_tag,$stack,$tag_ref) = @_;
    my @tags=@{$tag_ref};

    $current_tag++;
    push (@tags,$self);

    my $open_tag  = "    <polygon tag=\"$current_tag\" type=\"baby\">\n";
    my $close_tag = "    <polygon/>\n";

    $stack .= $open_tag;
    foreach my $element ($self->eachline()){
        ($current_tag,$stack,$tag_ref)=$element->render($current_tag,$stack,\@tags);
        @tags=@{$tag_ref};
    }
    $stack .= $close_tag;

    return ($current_tag,$stack,\@tags);
}


1;  # says use was ok

