=head1 NAME

Session

=head1 DESCRIPTION

The Session object tracks what model and view are for each user session in
the gui. This code should only be used from the SessionList object.

=head1 AUTHOR

B<Alex Bateman> Email agb@sanger.ac.uk

=cut

#
# Perl Module for Session
#
# Cared for by Alex Bateman <agb@sanger.ac.uk>
#

package Session;

use vars qw($AUTOLOAD @ISA @EXPORT_OK);
use Exporter;
use Carp;
use strict;
use Gui::SessionList;

#
# Place functions/variables you want to *export*, ie be visible from the caller package into @EXPORT_OK
#

@EXPORT_OK = qw();

#
# @ISA has our inheritance.
#

@ISA = ( 'Exporter' );

#######
# new #
#######

=head2 new

Create new session object.

    $session = new Session($model,$view,$name);

=cut

sub new {
    my ($class,$model,$view,$name)=@_;

    my $self = {
    };

    bless $self, $class;

    $self->model($model);
    $self->view($view);
    $self->name($name);

    return $self;
}

#########
# model #
#########

=head2 model

Get set method

    $session->model($model);

=cut

sub model {
    my ($self,$model)=@_;

    if (! $model){
	$model=$self->{'model'};
    } else {
	$self->{'model'}=$model;
    }
    return $model;
}

########
# view #
########

=head2 view

Get set method

    $session->view($view);

=cut

sub view {
    my ($self,$view)=@_;

    if (! $view){
	$view=$self->{'view'};
    } else {
	$self->{'view'}=$view;
    }
    return $view;
}

########
# name #
########

=head2 name

Get set method

    $session->name($name);

=cut

sub name {
    my ($self,$name)=@_;

    if (! $name){
	$name=$self->{'name'};
    } else {
	$self->{'name'}=$name;
    }
    return $name;
}


1;  # says use was ok
__END__






