
#
# Perl Module for Postscript
#
# Cared for by Alex Bateman <agb@sanger.ac.uk>
#

package Postscript;

use vars qw($AUTOLOAD @ISA @EXPORT_OK);
use Exporter;
use View;
use Text::Wrap;

#
# Place functions/variables you want to *export*, ie be visible from the caller package into @EXPORT_OK
#

@EXPORT_OK = qw();

#
# @ISA has our inheritance.
#

@ISA = ('Exporter','View');

#######
# new #
#######

=head1 new

Creates new postscript object.

$view = new Postscript($model,\*FILEHANDLE);

=cut

sub new {
    my ($class,$model,$filehandle) = @_;

    my $self = new View($model,$filehandle);

    bless $self, $class;

    # Attach view object to model so it can notify it of updates.
    $model->attach_view($self);
    return $self;
}




##########
# header #
##########

=head1 header

Prints postscript header to FILEHANDLE

    $output ->header(\*FILEHANDLE);

=cut

sub header {
  my ($self,$FILEHANDLE,$parameters) = @_;

  my $text=1;
  my $book=1;

  if ($book){$text=0;}

   my ($page,$title);

  if ($book){

      # NASTY BIT OF CODE TO GET TITLE AND PAGE NUMBERS FROM XML STACK
      my ($stack,$tag_ref)=$self->{'model'}->render();
      my @lines=split /^/,$stack;
      
      foreach my $line (@lines){
	  my ($tag,$rest);
	  if ($line =~ /\<(\S+)\s+(.*)/){
	      $tag=$1;
	      $rest=$2;
	  }
	  
	  if ($tag eq 'tiling'){
	      if ($rest =~ /page=\"(\d+)\"/){
		  $page = $1;
	      }
	      
	      if ($rest =~ /title=\"([^\"]*)\"/){
		  $title = $1;
	      }
	  }
      }
  }
  

  print $FILEHANDLE <<"EOF";
%!PS-Adobe-3.0 EPSF-3.0
%%Creator: Origami by Alex Bateman
%%Title: Origami Tessellation crease pattern
%%Pages: (atend)
%%PaperSize: A4
%%BoundingBox: 25 25 575 575
%%EndComments
%%EndProlog
%%Page: 1 1

/m {moveto} bind def
/l {lineto} bind def
/s {stroke} bind def
/f {fill} bind def
/c {0 360 arc} bind def
/sg {setgray} bind def
/sb {0 setgray} bind def
/fl {2 setlinewidth} bind def
/tl {1 setlinewidth} bind def

%% black
/col0 {0.000 0.000 0.000 srgb} bind def
%% gray
/col1 {0.500 0.500 0.500 srgb} bind def
%% red
/col2 {1.000 0.000 0.000 srgb} bind def
%% turquoise
/col3 {0.000 0.560 0.560 srgb} bind def
%% white
/col4 {1.000 1.000 1.000 srgb} bind def
%% dark blue
/col5 {0.000 0.000 0.560 srgb} bind def
%% yellow
/col6 {1.000 1.000 0.000 srgb} bind def
%% lightblue
/col7 {0.530 0.810 1.000 srgb} bind def
%% green
/col8 {0.000 0.820 0.000 srgb} bind def
%% gold
/col9 {1.000 0.840 0.000 srgb} bind def

/cp {closepath} bind def
/gr {grestore} bind def
/gs {gsave} bind def
/sa {save} bind def
/rs {restore} bind def
/rm {rmoveto} bind def
/n {newpath} bind def
/sh {show} bind def
/slw {setlinewidth} bind def
/srgb {setrgbcolor} bind def
/sd {setdash} bind def
/ff {findfont} bind def
/sf {setfont} bind def
/scf {scalefont} bind def
/sw {stringwidth} bind def
/shd {dup dup currentrgbcolor 4 -2 roll mul 4 -2 roll mul 4 -2 roll mul srgb} bind def
/ef {eofill} bind def
/N {/y0 y0 20 sub def x0 y0 moveto} bind def

EOF

my $scale=1; # Scaling for tessellation in page.
if ($book){

    # NASTY BIT OF CODE TO GET TITLE AND PAGE NUMBERS FROM XML STACK
    my ($stack,$tag_ref)=$self->{'model'}->render();
    my @lines=split /^/,$stack;

    my ($page,$title);
    foreach my $line (@lines){
	my ($tag,$rest);
	if ($line =~ /\<(\S+)\s+(.*)/){
	    $tag=$1;
	    $rest=$2;
	}

	if ($tag eq 'tiling'){
	    if ($rest =~ /page=\"(\d+)\"/){
		$page = $1;
	    }

	    if ($rest =~ /title=\"([^\"]*)\"/){
		$title = $1;
	    }
	}
    }

    banner($self,$FILEHANDLE,$title,35,'col3','col4');
    book_blurb($self,$FILEHANDLE,50,100,$page,0,16);
    sidebar($self,$FILEHANDLE,'col7');
    number_page($self,$FILEHANDLE,275,20,$page);
    $scale=0.85;
}


# Print out parameters to postscript
  if ($parameters){
    my %parameters=%{$parameters};
    my ($element);
      print $FILEHANDLE <<"EOF";
/Times-Roman findfont
14 scalefont
setfont
25 700 m
EOF

  print $FILEHANDLE "(Parameters: ";
    foreach $element (sort keys %parameters){
      print $FILEHANDLE "$element $parameters{$element} ";
    }
    print ") show\n";
  }

  if ($text){
    print $FILEHANDLE <<"EOF";
/Times-Roman findfont
12 scalefont
setfont
350 10 m
(Copyright )show  
/Symbol findfont 
14 scalefont
setfont 
(\343) show
/Times-Roman findfont
14 scalefont
setfont
(Alex Bateman 1997-2005) show
EOF
  }


if ($book){print $FILEHANDLE "0 180 translate\n"}

print $FILEHANDLE <<"EOF";
%% Bounding box
/clippath
{n
25 225 m
25 575 l
575 575 l
575 25 l
25 25 l
cp} def
$scale $scale scale
clippath clip
clippath s

300 300 translate
EOF

  if ($parameters->{'scale'}){
    print $FILEHANDLE  1/$parameters->{'scale'}," ",1/$parameters->{'scale'}," scale\n";
  }

}

# Routine to write header for pages 2 onwards
sub page_header {
    my ($self,$page,$fh) = @_;

    print $fh <<EOF;
\%\%Page: $page $page
\%\%BeginPageSetup
<< /PageSize [595 842] >> setpagedevice

1 1 scale
\%\%EndPageSetup
sa
EOF
}

sub book_blurb {
    my ($self,$fh,$x,$y,$dpage,$center,$fontsize) = @_;

    my $x0 = $x;

    #seek( DATA, 0, 0 );

    $Text::Wrap::columns = 60;
    $/ = "\n\n";
    my @data;
    open (DATA, "BLURB") or die "Cannot open file BLURB!";
    while(<DATA>) {
        last if( /\# page $dpage\s*$/ );
    }
    while( <DATA> ) {
        last if( /\# page/ );
        s/\(/\\\(/g;
        s/\)/\\\)/g;
        push( @data, $_ );
    }

    my $suffix = ") col0 sh";
    if( $center ) {
        $suffix = ") dup sw pop 2 div neg 0 rmoveto col0 sh";
    }

    my $string = sprintf( "%s", fill( "\(", "\(", @data ) );
    $string .= "\n";
    $string =~ s/(.)\n/$1$suffix\n/g;
    my $count = $string =~ s/\n/ N\n/g;
    my $y0 = $count*10;
    $y0=$y0+$y;

    print $fh <<EOF;
/Helvetica ff $fontsize scf sf
gs 
/x0 $x0 def
/y0 $y0 def
x0 y0 m 
EOF

    print $fh $string;
    print $fh "gr\n\n";

    $/ = "\n";
}



sub _ps_logo {
    my ($self,$fh) = @_;

    print $fh <<EOF;
gsave
0.2 0.2 scale
2750 3750 translate
n
10 setlinewidth
col4
140.19 82.47 m 87.53 140.19 l s
82.47 29.81 m 140.19 82.47 l s
29.81 87.53 m 82.47 29.81 l s
87.53 140.19 m 29.81 87.53 l s
-29.81 -87.53 m -82.47 -29.81 l s
-82.47 -29.81 m -140.19 -82.47 l s
-140.19 -82.47 m -87.53 -140.19 l s
-87.53 -140.19 m -29.81 -87.53 l s
29.81 -82.47 m 82.47 -140.19 l s
82.47 -140.19 m 140.19 -87.53 l s
140.19 -87.53 m 87.53 -29.81 l s
87.53 -29.81 m 29.81 -82.47 l s
-140.19 87.53 m -87.53 29.81 l s
-87.53 29.81 m -29.81 82.47 l s
-29.81 82.47 m -82.47 140.19 l s
-82.47 140.19 m -140.19 87.53 l s
-140.19 -82.47 m -87.53 29.81 l s
-82.47 -29.81 m 29.81 -82.47 l s
29.81 -82.47 m 82.47 29.81 l s
82.47 29.81 m -29.81 82.47 l s
82.47 -140.19 m -29.81 -87.53 l s
140.19 82.47 m 87.53 -29.81 l s
-29.81 82.47 m -82.47 -29.81 l s
-82.47 140.19 m 29.81 87.53 l s
1 setlinecap 1 setlinejoin
cp
grestore

EOF
}

sub number_page {
    my ($self,$fh,$x,$y,$n) = @_;
    
    print $fh <<EOF;
/Helvetica ff 16 scf sf
gs
$x $y m gs ($n) dup sw pop 2 div neg 0 rmoveto col0 sh gr
gr
EOF

}


##########
# banner #
##########
=head1 footer
Prints banner footer to FILEHANDLE

    $output->banner(\*FILEHANDLE,$text,$fontsize,$colour,$text_colour);
=cut
sub banner {
    my ($self,$fh,$text,$fontsize,$colour,$text_colour) = @_;

    my( $x1, $x2 ) = ( 15, 580 );
    my( $y1, $y2 ) = ( 720, 780 );
    my $y3 = $y1 + 20;

    print $fh <<EOF;




n $x1 $y1 m $x2 $y1 l $x2 $y2 l $x1 $y2 l cp gs $colour 1.00 shd ef gr gs $colour s gr 
/Helvetica ff $fontsize scf sf
EOF

print $fh "275 $y3 m gs ($text) dup sw pop 2 div neg 0 rmoveto $text_colour sh gr\n";
    _ps_logo($self,$fh);

}

###########
# sidebar #
###########
=head1 sidebar
Prints sidebar to FILEHANDLE

    $output->banner(\*FILEHANDLE,$colour);
=cut
sub sidebar {
    my ($self,$fh,$colour) = @_;

    my( $x1, $x2 ) = ( 520, 580 );
    my( $y1, $y2 ) = ( 10, 719.3 );
    print $fh <<EOF;

n $x1 $y1 m $x2 $y1 l $x2 $y2 l $x1 $y2 l cp gs $colour 1.00 shd ef gr gs $colour s gr 
EOF

}


##########
# footer #
##########
=head1 footer
Prints postscript footer to FILEHANDLE

    $output ->footer(\*FILEHANDLE);
=cut
sub footer {
  my ($self,$FILEHANDLE) = @_;

  my ($text)=0;

  print $FILEHANDLE <<"EOF";
1 setlinecap 1 setlinejoin
s showpage
EOF

#  exit (0);
}

# Write footer for page 2 onwards
sub _ps_page_footer {
    my ($self,$fh) = @_;
    print $fh "rs\nshowpage\n";
}



########
# body #
########

=head1 body

Print body of postscript

    $output->body($FILEHANDLE);

=cut

sub body {
    my $self = shift @_;
    my $FILEHANDLE = shift @_;



    my %args = @_;  # This has other parameters that can be passed to Model
                    # Such as whether to render a light pattern

    my ($stack,$tag_ref)=$self->{'model'}->render(%args);

    my $centres=$self->{'centres'};

    my $light;
    if ($args{'light'}){
        $light=$args{'light'};
        warn "Should try and render light pattern with resolution $light";
    }

    my @lines=split /^/,$stack;

    foreach my $line (@lines){
	my ($tag,$rest);
	if ($line =~ /\<(\S+)\s+(.*)/){
	    $tag=$1;
	    $rest=$2;
	}

	if ($tag eq 'line'){
	    if ($rest =~ /crease=\"(\S)\" x1=\"(\S+)\" y1=\"(\S+)\" x2=\"(\S+)\" y2=\"(\S+)\"/){
		if ($1 eq 'm'){
		    print $FILEHANDLE "tl $2 $3 m $4 $5 l s\n";
		} else {
		    print $FILEHANDLE "fl $2 $3 m $4 $5 l s\n";
		}
	    }
	}

	# Show centre of rotation if switched on
	if ($tag eq 'centre' and $centres){
	    if ($rest =~ /x=\"(\S+)\" y=\"(\S+)\"/){
		my $radius = 2;
		if ($rest =~ /NaNQ/){
		    die "REST $rest";
		}

		print $FILEHANDLE int($1)," ", int($2)," $radius c s\n";
	    }
	}


        if ($light){
            if ($tag eq 'polygon'){
                if ($rest =~ /layers=\"(\d+)\" x1=\"(\S+)\" y1=\"(\S+)\" x2=\"(\S+)\" y2=\"(\S+)\" x3=\"(\S+)\" y3=\"(\S+)\" x4=\"(\S+)\" y4=\"(\S+)\"/){
		    my $x1=int $2;
		    my $y1=int $3;
		    my $x2=int $4;
		    my $y2=int $5;
		    my $x3=int $6;
		    my $y3=int $7;
		    my $x4=int $8;
		    my $y4=int $9;


		    print $FILEHANDLE  1-(0.1*$1)," sg ",
		    $x1," ",$y1," m ",
		    $x2," ",$y2," l ",
		    $x3," ",$y3," l ",
		    $x4," ",$y4," l ",
		    $x1," ",$y1," l f s sb\n";
                }
            }
        }
    }
}

1;  # says use was ok


__DATA__

# page 1

Copyright (c) 2005 by Alex Bateman

All rights reserved.  The patterns in this E-book may not be resold or
redistributed in any form without written permission from the
author.

# page 2

The picture above is of a folded origami tessellation. A style of
folding developed by Yoshide Momatani, Shuzo Fujimoto and more
recently by Chris Palmer.  If they are folded from transparent paper
such as tracing paper (UK) or vellum (US) they show spectacular
patterns when held up to a bright light.

# page 3

So you would like to fold some Origami Tessellations. This book
contains a collection of designs I have made over the past ten years.
I have not used standard origami notation for these models, it just is
not possible to provide step-by-step instructions. To fold them
successfully I suggest the following method.


1. Photocopy the pattern onto a transparent or thin paper. 

2. Fold the pattern on a flat surface. 

3. Fold ALL the thin lines as valley folds. Then turn the the paper over and fold ALL the thick lines as valley folds (These thick lines might be visible through non-transparent paper). 

4. Gently include all the creases you have made, this makes parts of the model twist. 

5. Be patient. 

6. Flatten the model completely and hold up to the light to see the final pattern. 


# page 4

This is a simple example to help you understand how to collapse
a simple twist fold. Later on we will combine these simple elements
or tiles into an origami tessellation.

# page 5

This twist fold is called an open back twist fold.

# page 6

This twist fold is called an iso area twist fold. Some layers go in
front of the central hexagon whereas others go behind.

# page 7

This is called an closed back twist fold.


# page 8

This is the most complex twist fold you will see in this book.  So if you
can get this right you have all the basic skills you will need.

# page 9

Here is the iso area version of an octagon twist. 

# page 10

Now we put four twist folds together to make our first real tessellation.

# page 11

Make sure to crease all the thick lines first.  Then turn the paper over
and crease all the thin lines.  This will help to stop you getting
confused and making some creases in the wrong direction.


# page 12

The book Patterns and Tilings by Grunbaum and Shephard
(ISBN:0716719983) is the essential guide to tessellations. It is an
extraordinary book.  that gave me a lot of inspiration.


# page 13

You can find out more about Origami Tessellation at the following URL

www.sanger.ac.uk/Users/agb/Origami/Tessellation# page 14


# page 15

When you hold your tessellation up to a bright light you see the full beauty
of the model in the light pattern.  The various number of layers overlapping
back and forth give rise to completely unexpected shapes. These patterns are
why I fold tessellations.

# page 16

Here is a practice run for the hexagon swirl

# page 17

The hexagon swirl is my all time favourite tessellation.  I love the
way the network of triangles seem to be sitting above the rest of the paper.
