
#
# Perl Module for Scalable Vector Graphic output
#
# Cared for by Alex Bateman <agb@sanger.ac.uk>
#

package SVG;

use vars qw($AUTOLOAD @ISA @EXPORT_OK);
use Exporter;
use View;

#
# Place functions/variables you want to *export*, ie be visible from the caller package into @EXPORT_OK
#

@EXPORT_OK = qw();

#
# @ISA has our inheritance.
#

@ISA = ('Exporter','View');

#######
# new #
#######

=head1 new

Creates new SVG object.

$view = new SVG($model,\*FILEHANDLE);

=cut

sub new {
    my ($class,$model,$filehandle) = @_;

    my $self = new View($model,$filehandle);

    bless $self, $class;
    # Attach view object to model so it can notify it of updates.
    $model->attach_view($self);

    return $self;
}




##########
# header #
##########

=head1 header

Prints SVG header to FILEHANDLE

    $output ->header(\*FILEHANDLE);

=cut

sub header {
  my ($self,$FILEHANDLE,$parameters) = @_;

  my $text=1;
    die "HERE WE ARE";

  print $FILEHANDLE <<"EOF";
<?xml version="1.0" ?> 
<svg width="500" height="500">
  <title>Tess Origami</title>   
  <desc>SVG format origami tessellation</desc> 
  
EOF

}

##########
# footer #
##########
=head1 footer
Prints postscript footer to FILEHANDLE

    $output ->footer(\*FILEHANDLE);
=cut
sub footer {
  my ($self,$FILEHANDLE) = @_;

  my ($text)=0;

  print $FILEHANDLE <<"EOF";
  </svg>
EOF

#  exit (0);
}

########
# body #
########

=head1 body

Print body of postscript

    $output->body($FILEHANDLE);

=cut

sub body {
    my $self = shift @_;
    my $FILEHANDLE = shift @_;

    my %args = @_;  # This has other parameters that can be passed to Model
                    # Such as whether to render a light pattern

    my ($stack,$tag_ref)=$self->{'model'}->render(%args);

    my $centres=$self->{'centres'};

    my $light;
    if ($args{'light'}){
        $light=$args{'light'};
        warn "Should try and render light pattern with resolution $light";
    }

    my @lines=split /^/,$stack;

    foreach my $line (@lines){
	my ($tag,$rest);
	if ($line =~ /\<(\S+)\s+(.*)/){
	    $tag=$1;
	    $rest=$2;
	}


	if ($tag eq 'line'){
	    if ($rest =~ /crease=\"(\S)\" x1=\"(\S+)\" y1=\"(\S+)\" x2=\"(\S+)\" y2=\"(\S+)\"/){
		if ($1 eq 'm'){
		    print $FILEHANDLE "  <line x1=\"$2\" y1=\"$3\" x2=\"$4\" y2=\"$5\" style=\"stroke: black;\"/>\n";
		} else {
		    print $FILEHANDLE "  <line x1=\"$2\" y1=\"$3\" x2=\"$4\" y2=\"$5\" stroke-width=\"2\" style=\"stroke: black;\"/>\n";
		}
	    }
	}

	# Show centre of rotation if switched on
	if ($tag eq 'centre' and $centres){
	    if ($rest =~ /x=\"(\S+)\" y=\"(\S+)\"/){
		my $radius = 2;
		if ($rest =~ /NaNQ/){
		    die "REST $rest";
		}

		print $FILEHANDLE int($1)," ", int($2)," $radius c s\n";
		my $cx=int($1);
		my $cy=int($2);

		print $FILEHANDLE "   <circle cx=\"$cx\"> cy=\"$cy\" r=\"$radius\" stroke=\"black\"\n";
	    }
	}


        if ($light){
            if ($tag eq 'polygon'){

		die "Oh come on!  Alex is too lazy to implement light patterns in SVGformat.  Its only about three minutes work!\n";
                if ($rest =~ /layers=\"(\d+)\" x1=\"(\S+)\" y1=\"(\S+)\" x2=\"(\S+)\" y2=\"(\S+)\" x3=\"(\S+)\" y3=\"(\S+)\" x4=\"(\S+)\" y4=\"(\S+)\"/){
		    my $x1=int $2;
		    my $y1=int $3;
		    my $x2=int $4;
		    my $y2=int $5;
		    my $x3=int $6;
		    my $y3=int $7;
		    my $x4=int $8;
		    my $y4=int $9;


		    print $FILEHANDLE  1-(0.1*$1)," sg ",
		    $x1," ",$y1," m ",
		    $x2," ",$y2," l ",
		    $x3," ",$y3," l ",
		    $x4," ",$y4," l ",
		    $x1," ",$y1," l f s sb\n";
                }
            }
        }
    }
}





1;  # says use was ok

__END__
