#! /usr/local/bin/perl -w

# command line interface for polygon objects

use Model;
use View;
use strict;
use Getopt::Long;

# Parameters are all global
my ($unfolded,$geometry,$ratio,$angle,$bbox_scale,$x,$y,$scale,$show_centres,$profile,$help,$output_file,$svg,$rotate,$weave,$render,$opaque,$reverse,$maekawa,$n,$iso_area,$page,$title);
&GetOptions('unfolded!'  => \$unfolded,
	    'geometry=i' => \$geometry,
	    'render=s'   => \$render,
	    'help!'      => \$help,
	    'reverse!'   => \$reverse,
            'output=s'   => \$output_file,
            'svg!'       => \$svg,
            'bbox=s'     => \$bbox_scale, # Scale bounding box further!
            'ratio=s'    => \$ratio,
            'title=s'    => \$title, # Title for tessellation
            'scale=s'    => \$scale,
            'x=s'        => \$x,
            'y=s'        => \$y,
            'rotate=s'   => \$rotate,     # Rotates tiling by degrees!
            'weave=s'    => \$weave,      # Specifies weave angle
            'opaque!'    => \$opaque,
            'maekawa!'   => \$maekawa,
            'centres!'   => \$show_centres,
            'angle=s'    => \$angle,
	    'n=s'        => \$n,
	    'page=s'     => \$page,       # Stores page number for book
            'title=s'    => \$title,      # Stores page title for book
	    'iso_area!'  => \$iso_area);

if (! $geometry or $help){&help;}


&to_do();
&do();

sub to_do {
    # List of known bugs to fix
    print STDERR <<"EOF";
This program has known bugs!

1)   ./command.pl -geometry 1 -ratio 1 -angle -20 -opaque

      Found May 26th
      -opaque option is not using a correct algorithm yet!
      The above exmple does not give correct folded pattern.

3)    ./command.pl -geometry 3 -ratio 1 -angle -150 -weave -30

      Found May 26th
      Crease pattern needs to change based on input parameters!
      This crease pattern will not fold.

4)    ./command.pl -geometry 16 -ratio 1 -angle 30 -maekawa

      Found May 27th
      Double decompose routine fails to give correct crease assignment?!

5)    ./command.pl -geometry 17 -ratio 1 -angle 30 -maekawa

       Incorrect crease assignment, do not understand yet. Seems that
       alternate is not correct in multiple_rectangle_grid.  

FIXED BUGS

1)    ./command.pl -geometry 10 -ratio 1 -angle 20 -maekawa -bbox 2 -scale 0.5

      Found and fixed May 27th 2001.

      Appears to be a bug with crease assignment of babies for this tiling!
      This appears to be due to way crease assignment is done for babies, which
      depends on direction points are defined around triangles!

2)    ./command.pl -geometry 6 -ratio 1 -angle 20

      Found May 26th Fixed Jan 16th 2002 
      Severe bug geometry 6 will not work.  Does not recognise when
      an expanded bounding box contains a polygon and enters an
      infinite loop.

      Appears that contains_point2 method using odd-even rule was
      flawed.  Have not found bug in that routine, but switched
      over to use contains_point method which seems to have cleared
      up the problem.


EOF
}

sub help {
    print STDERR <<"EOF";

$0 <arguments>

Arguments:
         -geometry <integer>  Choose geometry (see list below)
         -angle <angle>       Specify pleat angle
         -ratio <value>       Pleat ratio

         -render <pixel size> Render light pattern
         -unfolded            Gives un-origamified tiling
         -reverse             Changes parity of all creases
	 -help                Get usage
         -output <filename>   Output file format (postscript default)
         -svg                 Use SVG format output file
         -bbox <value>        Scale bounding box further

         -scale <value>       Scale for output
         -x <value>           Translate output
         -y <value>           Translate output
         -rotate <angle>      Rotate output

         -centres             Show centres of rotation for polygons

         -n                   Number of sides for simple twist geometry 99
	 -iso_area            Make creases Iso-area for simple twist geometry 99

         ADVANCED ARGUMENTS

         These should be used with care, as they are buggy or just for
         my own use.

         -weave <angle>       Used in weave tessellations
         -opaque              Trys to give pattern folded from opaque paper
         -maekawa             Used when assigning crease pattern to new geometries,
                              will circle vertices that break maekawas theorem


Geometries:
   1   4.4.4.4
   2   6.6.6
   3   3.3.3.3.3.3
   4   4.8.8
   5   3.6.3.6
   6   3.4.6.4
   7   4.6.12
   8   3.4.3.12; 3.12.12
   9   decomposed d4.d4.d4.d4
   10  Star
   11  Star
   12  Star
   13  4.d8.d8
   14  Star decomposed
   15  Crystal c6.c6.c6
   16  Double decomposed 4.4.4.4
   17  d4.d8.d8
   18  Crystal polygon
   19  Kite
   20  Spiral
   21  Voderbergs spiral (broken)
   22  3.12.12
   23  3.3.3.4.4
   24  Square weave
   25  nothing
   26  3.d3.3.d3.3.d3
   27  3.3.3.3.3.3; 3.3.4.3.4
   28  3.3.3.3.3.3; 3.3.4.12
   29  Rosette 1
   30  Rosette 2
   31  Windmill
   32  3.6.3.6; 3.3.6.6
   33  3.3.3.3.3.3; 3.3.6.6
   34  3.3.4.3.4
   35  decomposed 9.9.6.4pi9 version 2
   36  3.4.4.6;3.6.3.6 version 1
   37  3.4.4.6;3.6.3.6 version 2
   38  Squarex 1
   39  Squarex 2
   40  Squarex 3
   41  Squarex 4
   99  Simple twist
EOF
    exit 1;
}

sub do {

    # Make new Model object
    my $model;
    my %args;

    if ($bbox_scale){
	$args{'bbox_scale'} = $bbox_scale;
    }
    if ($n){
	$args{'rot_n'} = $n;
    }
    if ($iso_area){
	$args{'iso_area'} = 1;
    }

    if (! $weave){
	$model = new Model($geometry,'twist',%args);
    } else {
	$model = new Model($geometry,'weave',%args);
    }


    my %parameters;
    if ($title){
	$model->title($title);
    }
    if ($page){
	$model->page($page);
    }

    open (O1, "> tiling.ps")||die "Can't write to file tiling.ps\n";

    # Make a View::Postscript object for underlying tiling
    my $view;
    if (! $svg){
	$view = new Postscript($model,\*O1,\%parameters);
	warn "VIEW $view\n";
    } else {
	$view = new SVG($model,\*O1);
	warn "VIEW $view\n";
    }


    if (! $output_file){$output_file='foo.ps';}
    open (O2, "> $output_file")||die "Can't write to file $output_file\n";


    if (! $unfolded){
	$view->show('creases');
	$model->vertexify();

	if ($reverse){
	    $model->reverse();
	}

	if ($scale){
	    $model->scale($scale,0,0);
	}

	if ($rotate){
	    $model->rotate($rotate,0,0);
	}
	

	if ($x or $y){
	    $model->translate($x,$y);
	}

	
	# Make a View::Postscript object
	my $view2;
	if (! $svg){
	    $view2 = new Postscript($model,\*O2);
	} else {
	    $view2 = new SVG($model,\*O2);
	}

	if ($show_centres){
	    $view2->show_centres('1');
	}

	if ($render){ # Light pattern
	    $view2->light($render);
	}

	# View will get updated at this point and postscript will get printed
	$view2->show('creases');

	$model->transform($ratio,$angle,$weave); 
    }


    $model->max_angle();




#    if ($maekawa){
#	###########################################
#	# Check points that don't satisfy maekawa #
#	###########################################
#	my @points=$model->maekawa();
#	foreach my $point (@points){
#	    $output->add($point);
#	}
#    }

#    if ($render){ # Light pattern
#	$output->postscript(\*O,$model,'render', 'render' => $render);
#    } elsif ($opaque) {
#	$output->postscript(\*O,$model,'opaque');
#    }else {
#	$output->postscript(\*O,$model,'standard');
#    }
}


__END__









