#! /usr/local/bin/perl5.6.0 -w

use strict;
use Tk;
use Model;
use View;
use Canvas;
use Getopt::Long;
use Gui::SessionList;
use Gui::Session;
use Carp;

# NEW SINCE VERSION TESS 1.1
#
# 1) Added simple twist geometry
# 2) Added rosette geometries
# 3) Added delete action to gui.pl
# 4) Added multiple sessions editing
# 5) Added option to unorigamify tessellations
# 6) Fixed bug with displaying pleat angle 2 when selecting weave tilings
# 7) Tidied up look of gui
#
# NEW SINCE VERSION TESS 1.2
#
# 1) Added Tiling description and folding state to interface
# 2) Added many new 2-uniform tilings
# 3) Added cascading menus for geometries
# 4) Added options to rotate and stretch tilings
#
#
# TO DO (IN NO PARTICULAR ORDER)
#
# 1)  Add random tilings
# 2)  Can I get colours right for decompositions?
# 3)  Default crease assignment for weave tilings are wrong when pleat angles > 90

##########################
# GLOBAL GUI PARAMETERS #
#########################

my $action;          # What type of action user will carry out when selecting tiles
my $state='none';    # Whether you have a tiling or a crease pattern 
my $description='no tiling selected';     # Description of tiling
my $crystal_param;
my $instar_param;
my $model;
my $view;
my $pleat_angle=20;
my $pleat_angle2=10;
my $pleat_ratio=1;
my $canvas_scale=1;
my $rotation=0; # Rotation of tiling
my $stretch=1;  # stretch of tiling.  1 means no stretch
my $session_list;
my $centre_state=0;
my $fill_state=0;
my $light_state=0;
my $weave_frame;   # Whether or not user is looking at a weave geometry
my @random_args;

&initialise_display; # Set the above parameters

my $mw = MainWindow->new;
$mw->title("Tess: Origami tessellations for the masses");

my $menuframe=$mw->Frame(-relief => 'groove')->pack(-side => 'top',
						    -fill => 'x');


my $mbutton1 = $menuframe->Menubutton(-text=>"File", -relief => 'raised', 
				      -menuitems =>[['command' => "Save as pdf", -command => \&print_to_pdf_window],
						    ['command' => "Save as postscript", -command => \&print_to_file_window],
						    ['command' => "Exact options", -command => \&exact_options_window],
						    
						    ['command' => "Exit",  -command => sub {exit},]
						   ])->pack(-side => 'left');

my $mbutton2 = $menuframe->Menubutton(-text=>"Geometry", -relief => 'raised',-tearoff=>0,
				      -menuitems =>[['command' => "Simple twist",
						     -command => sub {&twist_window()}],
						    "-",
						    [Cascade => "ARCHIMEDEAN TILINGS", -tearoff=>0, -menuitems =>
						     [
						      [Button => "4.4.4.4", -command => sub {&do_geometry("1",'twist')}],
						      [Button => "6.6.6", -command => sub {&do_geometry("2",'twist')}],
						      [Button => "3.3.3.3.3.3", -command => sub {&do_geometry("3",'twist')}],
						      [Button => "4.8.8", -command => sub {&do_geometry("4",'twist')}],
						      [Button => "3.6.3.6", -command => sub {&do_geometry("5",'twist')}],
						      [Button => "3.4.6.4", -command => sub {&do_geometry("6",'twist')}],
						      [Button => "4.6.12", -command => sub {&do_geometry("7",'twist')}],
						      [Button => "3.3.3.4.4", -command => sub {&do_geometry("23",'twist')}],
						      [Button => "3.3.4.3.4", -command => sub {&do_geometry("34",'twist')}],
						      [Button => "3.12.12", -command => sub {&do_geometry("22",'twist')}],
						      ]
						     ],
						    [Cascade => "2-UNIFORM TILINGS", -tearoff=>0, -menuitems =>
						     [
						      ['command' => "3.4.3.12;3.12.12",
						       -command => sub {&do_geometry("8",'twist')}],
						      ['command' => "3.3.3.4.4;4.4.4.4 v1",
						       -command => sub {&do_geometry("28",'twist')}],
						      ['command' => "3.3.3.4.4;4.4.4.4 v2",
						       -command => sub {&do_geometry("46",'twist')}],
						      ['command' => "3.6.3.6;3.3.6.6",
						       -command => sub {&do_geometry("32",'twist')}],
						      ['command' => "3.3.3.3.3.3;3.3.6.6",
						       -command => sub {&do_geometry("33",'twist')}],
						      ['command' => "3.4.4.6;3.6.3.6 v1",
						       -command => sub {&do_geometry("36",'twist')}],
						      ['command' => "3.4.4.6;3.6.3.6 v2",
						       -command => sub {&do_geometry("37",'twist')}],
						      ['command' => "3.4.3.12;3.12.12",
						       -command => sub {&do_geometry("47",'twist')}],
						      ['command' => "3.3.4.3.4;3.4.6.4",
						       -command => sub {&do_geometry("49",'twist')}],
						      ['command' => "3.3.3.4.4;3.4.6.4",
						       -command => sub {&do_geometry("48",'twist')}],
						      ['command' => "3.4.4.6;3.4.6.4",
						       -command => sub {&do_geometry("51",'twist')}],
						      ['command' => "3.3.3.3.3.3;3.3.4.12",
						       -command => sub {&do_geometry("52",'twist')}],
						      ],
						    ],[Cascade => "3-UNIFORM TILINGS", -tearoff=>0, -menuitems =>
						     [
						      ['command' => "3.3.3.4.4;3.3.4.3.4;3.4.6.4",
						       -command => sub {&do_geometry("50",'twist')}],
						      ],
						       ],
						    [Cascade => "ROTATIONAL TILINGS", -tearoff=>0, -menuitems =>
						     [
						      ['command' => "tower",-command => sub {&tower_window()}],
						      ['command' => "rosettes",-command => sub {&rosette_window()}],
						      ['command' => "log spiral",-command => sub {&spiral_window}],
						      ['command' => "voderberg spiral",-command => sub {&voderberg_window}],
						      ['command' => "kepler tiling",-command => sub {&do_geometry("45",'twist')}],
						      ],
						     ],
						    [Cascade => "MISC. TILINGS", -tearoff=>0, -menuitems =>
						     [
						      ['command' => "Windmill tiling",
						       -command => sub {&do_geometry("31",'twist')}],
						      ['command' => "9.9.6.4pi9 star tiling",
						       -command => sub {&do_geometry("10",'twist')}],
						      ['command' => "4.6.4pi.6.6 star tiling",
						       -command => sub {&do_geometry("11",'twist')}],
						      ['command' => "8.4pi.4.8.4pi.4 star tiling",
						       -command => sub {&do_geometry("12",'weave')}],
						      ['command' => "decomposed 4.4.4.4",
						       -command => sub {&do_geometry("9",'twist')}],
						      ['command' => "partial decomposed 4.8.8",
						       -command => sub {&do_geometry("13",'twist')}],
						      ['command' => "full decomposed 4.8.8",
						       -command => sub {&do_geometry("17",'twist')}],
						      ['command' => "partial decomposed 9.9.6.4pi9",
						       -command => sub {&do_geometry("14",'twist')}],
						      ['command' => "full decomposed 9.9.6.4pi9",
						       -command => sub {&do_geometry("35",'twist')}],
						      ['command' => "6.6.6 crystal",
						       -command => sub {&do_geometry("15",'twist')}],
						      ['command' => "3.6.3.6 crystal",
						       -command => sub {&do_geometry("44",'twist')}],
						      ['command' => "double decomposed 4.4.4.4",
						       -command => sub {&do_geometry("16",'twist')}],
						      ['command' => "kites",
						       -command => sub {&do_geometry("19",'twist')}],
						      ['command' => "squarex 1",
						       -command => sub {&do_geometry("38",'twist')}],	
						      ['command' => "squarex 2",
						       -command => sub {&do_geometry("39",'twist')}],
						      ['command' => "squarex 3",
						       -command => sub {&do_geometry("40",'twist')}],
						      ['command' => "squarex 4",
						       -command => sub {&do_geometry("41",'twist')}],
						      ['command' => "Butterfly",
						       -command => sub {&do_geometry("42",'twist')}],
						      ['command' => "Squave",
						       -command => sub {&do_geometry("43",'twist')}],
						      ],
						     ],
						    [Cascade => "WEAVE TILINGS", -tearoff=>0, -menuitems =>
						     [
						      ['command' => "4.4.4.4 weave",
						       -command => sub {&do_geometry("1",'weave')}],
						      ['command' => "3.3.3.3.3.3 weave",
						       -command => sub {&do_geometry("3",'weave')}],
						      ['command' => "3.6.3.6 weave",
						       -command => sub {&do_geometry("5",'weave')}],
						     ],
						     ],
						    ])->pack(-side => 'left');


my $mbutton3 = $menuframe->Menubutton(-text=>"Select", -relief => 'raised',
				      -menuitems =>[['command' => "Action", -command => sub {&action_window()}],
						    ])->pack(-side => 'left');
    
my $mbutton5 = $menuframe->Menubutton(-text=>"Session", -relief => 'raised', 
				      -menuitems =>[['command' => "New Session", -command => sub{
							&new_session();
						    }],"-",])->pack(-side => 'left');


my $tk_canvas = $mw -> Canvas (-width => 600,
			       -height => 600,
			       -takefocus => 0,
			       -relief => 'ridge',
			       -borderwidth => 5,
			       -background => 'white')->pack(-side => 'bottom');

&change_mode('move'); # Default mode


###########
# Options #
###########
    
my $outer_frame =$mw->Frame(-relief => 'ridge',-borderwidth => 2)->pack(-side => 'left', -expand => 1);
my $origamify_frame =$outer_frame->Frame()->pack(
						 #-side => 'top', -expand => 1
						 );

my $unorigamify_button; # Have to declare variables here or they can't be used in anonymous subroutine call!
my $origamify_button;

$unorigamify_button=$origamify_frame->Button(-text => "Show tiling", -state => "disabled", -command =>sub {

    $state=$model->state();
    $view->show('tiling');
    
    if ($state eq 'tiling'){
	return 0;
    }

    # If this button is pressed, disable it and reenable origamify button
    $unorigamify_button->configure(-state => 'disabled');
    $origamify_button->configure(-state => 'normal');


    if ($state eq 'folded' or $state eq 'unfolded'){
	$model->undo_vertexify();
	$model->state('tiling');
	$state=$model->state();

	if (&change()){
	    warn "Shouldn't be calling change as no model yet!";
	}
	&change_mode('move');
    }
})->pack(-side => 'left');

$origamify_button=$origamify_frame->Button(-text => "Show creases", -state => "disabled", -command =>sub {
    $state=$model->state();
    $view->show('creases');
    
    # If this button is pressed, disable it and reenable unorigamify button
    $origamify_button->configure(-state => 'disabled');
    $unorigamify_button->configure(-state => 'normal');


    if ($state eq 'tiling'){
	$model->vertexify(); # This indirectly calls notify_view
	$state=$model->state();
	if (&change()){
#	    warn "Shouldn't be calling change as no model yet!";
	}
#	&change();
	&change_mode('move');
    } elsif ($state eq 'folded' or $state eq 'unfolded'){
	#report("You have already vertexified this tiling. You can't do it again!");
	return 0;
    }
})->pack(-side => 'left');

my $show_frame = $outer_frame->Frame()->pack(
					     #-side => 'bottom', -expand => 1
					     );

my $centre_button=$show_frame->Checkbutton(-text => "Show Centres",
				   -variable => \$centre_state,
				   -command => sub {
				       if (! $model){
					   report("You must select a tiling from the geometry menu first!");
					   return 0;
				       } else {
					   $view->show_centres($centre_state);
					   $model->notify_view();
				       }
				   })->pack(-side => 'left');

my $fill_button=$show_frame->Checkbutton(-text => "Show Fill",
				 -variable => \$fill_state,
				 -command => sub {
				     if (! $model){
					 report("You must select a tiling from the geometry menu first!");
					 return 0;
				     } else {
					 $view->fill($fill_state);
					 $model->notify_view();
				     }
				 })->pack(-side => 'left');

my $light_button=$show_frame->Checkbutton(-text => "Light pattern",
				  -variable => \$light_state,
				  -command => sub {
				      if (! $model){
					  report("You must select a tiling from the geometry menu first!");
					  $light_state=0;
					  return 0;
				      }
				      
				      $state=$model->state();
				      if ($state eq 'tiling'){
					  report("You must Origamify a tiling first!");
					  $light_state=0;
					  return 0;
				      } elsif ($state eq 'unfolded'){
					  report("Rendering light pattern. Will take a short while");
					  $view->light($light_state);
					  $model->notify_view();

					  report("You must have a folded pattern, make pleat angle negative!");
					  $light_state=0;
					  return 0;
				      } elsif ($state eq 'folded'){
					  report("Rendering light pattern. Will take a short while");
					  $view->light($light_state);
					  $model->notify_view();
				      } else {
					  report ("Oops unreachable code executed! State is $state");
				      }
				  })->pack(-side => 'left');


my $desc_frame = $outer_frame->Frame()->pack(
					    #-side => 'bottom', -expand => 1
					    );

my $state_window = $desc_frame->Label(-textvariable => \$state,
				     -relief       => 'groove',
				     -width        => 8,
				     -background   => 'white',
				     #-pady => 6,
				     )->pack(-side => 'left');

my $description_window = $desc_frame->Label(-textvariable => \$description,
					   -relief       => 'groove',
					   -width        => 25,
					   -background   => 'white',
					   #-pady => 6,
					   )->pack(-side => 'left');

my $frame0 = $mw->Frame(-relief => 'ridge',-borderwidth => 2)->pack(-side => 'left', -expand => 1);
    
my $frame1 = $frame0->Frame(-relief => 'groove', -width => 20)->pack(-side => 'left', -expand => 1);
    
$frame1->Scale(-from => -180, 
	       -to => 180, 
	       -variable => \$pleat_angle,
	       -orient => 'horizontal', 
	       -label => 'pleat angle',
	       -command => sub {&change})->pack();

my $frame2 = $frame0->Frame(-relief => 'groove')->pack(-side => 'left',-expand => 1);  
$frame2->Scale(-from => 0.1, 
	       -to => 10, 
	       -variable => \$pleat_ratio,
	       -resolution => 0.1,
	       -orient => 'horizontal', 
	       -label => 'pleat ratio',
	       -command => sub {&change})->pack();

my $frame3 = $frame0->Frame(-relief => 'groove')->pack(-side => 'left',-expand => 1);
$frame3->Scale(-from => 0.1, 
	       -to => 10, 
	       -variable => \$canvas_scale,
	       -resolution => 0.1,
	       -orient => 'horizontal', 
	       -label => 'canvas scale',
	       -command => sub {&change_scale})->pack();



MainLoop;


sub add_weave_option {
    $weave_frame = $frame0->Frame(-relief => 'groove', -width => 20)->pack(-side => 'left',
									 -expand => 1);
    $weave_frame->Scale(-from => -180, 
		   -to => 180, 
		   -variable => \$pleat_angle2,
		   -orient => 'horizontal', 
		   -label => 'pleat angle 2',
		   -length => '200',
		   -command => sub {&change})->pack();
}


sub remove_weave_option {
  if ($weave_frame){
    $weave_frame->packForget();
  }
}

# This routine provides a window for more exactly specifying
# parameters
sub exact_options_window {
  my $options_window = MainWindow->new;

  my $frame0 = $options_window->Frame(-relief => 'ridge',-borderwidth => 2)->pack(-side => 'top',
										  -expand => 1);


  my $frame1 = $frame0->Frame(-relief => 'groove', -width => 20)->pack(-side => 'top',
								   -expand => 1);

  $frame1->Label(-text => 'pleat angle' )->pack(-side => 'left');
  my $angle_entry=$frame1->Entry(-textvariable => \$pleat_angle,
				 -width => 5,
				 -takefocus => 1)->pack(-side => 'right');
  $angle_entry->bind("<Key-Return>",\&change);

  my $frame7 = $frame0->Frame(-relief => 'groove', -width => 20)->pack(-side => 'top',
								     -expand => 1);
  $frame7->Label(-text => 'pleat angle 2' )->pack(-side => 'left');
  my $angle_entry2=$frame7->Entry(-textvariable => \$pleat_angle2,
				  -width => 5,
				  -takefocus => 1)->pack(-side => 'right');
  $angle_entry2->bind("<Key-Return>",\&change);

  my $frame2 = $frame0->Frame(-relief => 'groove')->pack(-side => 'top',-expand => 1);
  $frame2->Label(-text => 'pleat ratio' )->pack(-side => 'left');
  my $ratio_entry=$frame2->Entry(-textvariable => \$pleat_ratio,
				 -width => 5,
				 -takefocus => 1)->pack(-side => 'right');
  $ratio_entry->bind("<Key-Return>",\&change);

  my $frame3 = $frame0->Frame(-relief => 'groove')->pack(-side => 'top',-expand => 1);
  $frame3->Label(-text => 'canvas scale' )->pack(-side => 'left');
  my $scale_entry=$frame3->Entry(-textvariable => \$canvas_scale,
				 -width => 5,
				 -takefocus => 1)->pack(-side => 'right');
  $scale_entry->bind("<Key-Return>",\&change_scale);

  my $frame4 = $frame0->Frame(-relief => 'groove')->pack(-side => 'top',-expand => 1);
  $frame4->Label(-text => 'tiling rotation' )->pack(-side => 'left');
  my $rotation_entry=$frame4->Entry(-textvariable => \$rotation,
				 -width => 5,
				 -takefocus => 1)->pack(-side => 'right');
  $rotation_entry->bind("<Key-Return>",\&change_rotation);

  my $frame5 = $frame0->Frame(-relief => 'groove')->pack(-side => 'top',-expand => 1);
  $frame5->Label(-text => 'tiling stretch' )->pack(-side => 'left');
  my $stretch_entry=$frame5->Entry(-textvariable => \$stretch,
				 -width => 5,
				 -takefocus => 1)->pack(-side => 'right');
  $stretch_entry->bind("<Key-Return>",\&change_stretch);




  $options_window->Button(-text => "close", -command =>sub {$options_window->destroy;})->pack;
}

# This routine must be used to change mode, will change bindings etc.
#
# Mode can be one of: none
#                     move
#                     decompose
#                     crystal
#                     instar
sub change_mode {
  $action = shift @_;
  if (! defined $view){
      return 0;
  }

  my $canvas=$view->handle();

  # warn "CANVAS $canvas trying to change action to $action!\n";
  # Switch off all bindings
  $canvas->Tk::bind("<Button-1>","");
  $canvas->Tk::bind("<ButtonPress-1>","");
  $canvas->Tk::bind("<ButtonRelease-1>","");

  # Add new bindings
  if ($action eq 'none'){} # Do nothing!
  elsif ($action eq 'move'){
    $canvas->Tk::bind("<ButtonPress-1>", [\&drag, Ev('x'), Ev('y')]);
  } elsif ($action eq 'decompose' or $action eq 'crystal' or $action eq 'instar' or $action eq 'delete'){
    $canvas->Tk::bind("<Button-1>", [\&select_tile, Ev('x'), Ev('y'), $action]);
    warn "Action is now $action!\n";
  } else {
    &report ("Can't have an action of [$action]\n");
  }
}

sub do_geometry {
    my $geometry = shift @_;
    my $type     = shift @_;
    my %args=@_;


    # Check whether a Model already exists
    if ($model){
	report ("You cannot select a new geometry until the canvas is clear (Select New Session in See Session menu)");
	return 0;
    }

    # Can now disable the origamify button
    $origamify_button->configure(-state => 'normal');

    if ($type eq 'weave'){
	&add_weave_option();
    } else {
	if ($weave_frame){
	    &remove_weave_option();
	}
    }

    $model = new Model($geometry,$type,%args);
    $view=new Canvas($model,$tk_canvas);
    $view->show('tiling');

    if (! $session_list){
	$session_list=new SessionList();
    }
    $description=$model->description();
    my $session_name=$session_list->add($model,$view,$description);

    $state=$model->state();
    $description=$model->description();

    # Add session name to menu 
    $mbutton5->AddItems(["command" => $session_name,
			 -command => sub {&change_session($session_name)}]
			);
    $model->translate(600/2,600/2); # As a by product of this translation notify_view is called
                                    # and the tiling is displayed for the first time!

    &change_mode('move');
}


# Subroutine to make ready for a new session
sub new_session {
    $model="";
    $view->clear();
    $view="";
    $state='none';
    $description='no tiling selected';
    &remove_weave_option();
    &initialise_display();

    # Can now disable the origamify and unorigamify buttons
    $origamify_button->configure(-state => 'disabled');
    $unorigamify_button->configure(-state => 'disabled');

}

# Allows user to change between sessions
sub change_session {
    my $session_name=shift @_;

    &new_session();
    my $session=$session_list->get($session_name);

    $model=$session->model();
    $view=$session->view();

    # Get parameters from model and reset global gui variables
    my %parameters=$model->parameters();
    warn %parameters;
    $pleat_angle =$parameters{'pleat_angle'};
    $pleat_angle2=$parameters{'pleat_angle2'};
    $pleat_ratio =$parameters{'pleat_ratio'};
    $canvas_scale=$parameters{'absolute_scale'};

    # Also want to get view type parameters matched up such as centres, fill, light
    $centre_state=$view->show_centres();
    $fill_state  =$view->fill();
    $light_state =$view->light();

    # Check if weave option need to be added or removed
    my $type=$parameters{'type'};
    if ($type eq 'weave'){
	&add_weave_option();
    } else {
	if ($weave_frame){
	    &remove_weave_option();
	}
    } 
    $model->notify_view();
}

# This routine is called whenever user changes a parameter. This then
# calls the model and lets it know to notify the view object.

sub change {

    if (! $model){
	# This is called 2 times as scale bars are set up
	return 1;
    }

    $state=$model->state();
    if ($state eq 'tiling'){
      # Not sure why but I need to add this to get weave tilings to work.
      # Something to do with add_weave_option calling this routine on creation.
      return 1;
    }


    # Check if parameters have yet been assigned. If not assign them.
    if (! defined ($pleat_ratio) or ! defined ($pleat_angle) or ! defined ($pleat_angle2)){
	$pleat_ratio  = 1;
	$pleat_angle  = 20;
	$pleat_angle2 = 20;
    }
    #warn "calling transform $pleat_ratio,$pleat_angle,$pleat_angle2";
    $model->transform($pleat_ratio,$pleat_angle,$pleat_angle2);

    # Tiling is not yet a crease pattern, so can't change angles etc.
   # if ($state eq 'tiling' or $state eq 'none' or $state eq 'patch'){ 
#	$model->notify_view();
#	$model->transform();
#	return (0);


#    if ($state eq 'unfolded' and ! defined $pleat_angle){
#	$pleat_ratio  = 1;
#	$pleat_angle  = 20;
#	$pleat_angle2 = 20;
#	$model->transform($pleat_ratio,$pleat_angle,$pleat_angle2);
#    } elsif ($state eq 'unfolded'){
#	$model->transform($pleat_ratio,$pleat_angle,$pleat_angle2);
#    }

#    } elsif ($pleat_ratio > 0 && defined $pleat_angle){
#	$model->transform($pleat_ratio,$pleat_angle,$pleat_angle2);
#    } else {
#	$pleat_ratio  = 1;
#	$pleat_angle  = 20;
#	$pleat_angle2 = 20;
#	$model->transform($pleat_ratio,$pleat_angle,$pleat_angle2);
#    }
    $state=$model->state();
}

sub change_scale {
    if (! $model){
	#warn "Cannot change scale when no Model is made yet";
	return 1;
    }

    if ($canvas_scale>0){
	$model->scale($canvas_scale,300,300)
    } else {
	&report("You must specify a canvas scale");
    }
}

sub change_translate {
  my ($canvas,$x_translate,$y_translate)=@_;

  die "Is this routine ever used?";
  if (! defined $x_translate){die "In change_translate x_translate undefined"}
  if (! defined $y_translate){die "In change_translate y_translate undefined"}

  if ($rotation){
    $model->translate($x_translate,$y_translate);
  }
}

sub change_rotation {
    if (! $model){
	#warn "Cannot change rotation when no Model is made yet";
	return 1;
    }

    $model->rotate($rotation,300,300);
    $rotation=0;
}

sub change_stretch {
    if (! $model){
	#warn "Cannot change stretch when no Model is made yet";
	return 1;
    }

    $state=$model->state();

    if ($state eq "tiling"){
      $model->stretch($stretch);
      $stretch=1;
    } else {
      &report("You can only stretch when showing the tiling.  Either create a new tiling or select the Show Tiling button then try again. Note that value close to 1 look best e.g. 0.8 or 1.3.");
      $stretch=1;
    }
}

sub select_tile {
  my ($canvas,$x,$y,$mode) = @_;

  my $state=$model->state();
  if (! $state eq 'tiling'){
      report("You can only select tiles before Origamifying your tiling");
      return 0;
  }

  my $current_tag=$canvas->find("closest",$x,$y,10,"centre");

  my @tags=$canvas->gettags($current_tag);

  if (! defined $tags[0] or $tags[0] ne 'centre'){
    &report("You must click on the centre of a tile. Try switching on show centres!\nOr try increasing canvas scale");
    return 0;
  }


  # Need to convert this PerlTK canvas tag into a Model tag
  my $model_tag=$view->{'canvas_id2model_tag'}->{$current_tag};
  $model_tag--; # As polygon is one less than centre tag

  if (! $model_tag){
      die "Could not get a model tag from canvas tag $current_tag";
  }


  if ($mode eq 'None'){
    &report("No action is currently selected!");
    return 0;
  } elsif ($mode eq 'crystal'){
      my @crystal_param=split(/,/,$crystal_param);
      $model->decompose('crystal',$model_tag,@crystal_param);
  } elsif ($mode eq 'instar'){
      my ($angle,$shrink)=split(/,/,$instar_param);
      $model->decompose('instar',$model_tag,$angle,$shrink);
  } elsif ($mode eq 'decompose'){
      $model->decompose('decompose',$model_tag);
  } elsif ($mode eq 'delete'){
      $model->delete($model_tag);
  } else {
    &report("Your mode $mode is not in my list. Serious error!");
    return 0;
  }
  
  # Deselect old tag
  $canvas->itemconfigure("selected", -fill => "blue");
  $canvas->dtag("selected");
  
  # Select new tag
  $canvas->addtag("selected","withtag","$current_tag");
  $canvas->itemconfigure($current_tag, -fill => "yellow");
  $canvas->raise($current_tag);
}

# Need a window to input spiral parameters
sub spiral_window {
    my $spiral_window = MainWindow->new;
  
    my $m     = 10;
    my $rho   = 0.9;

    $spiral_window ->title("Spiral parameters window");

    my $frame1 = $spiral_window->Frame(-relief => 'groove')->pack(-side => 'left',-expand => 1);
    $frame1->Label(-text => 'Axial symmetry' )->pack(-side => 'left');
    my $m_entry=$frame1->Entry(-textvariable => \$m,
			       -width => 5,
			       -takefocus => 1)->pack(-side => 'right');
  
    my $frame2 = $spiral_window->Frame(-relief => 'groove')->pack(-side => 'left',-expand => 1);
    $frame2->Label(-text => 'Rho' )->pack(-side => 'left');
    my $rho_entry=$frame2->Entry(-textvariable => \$rho,
				 -width => 5,
				 -takefocus => 1)->pack(-side => 'right');
    
    $spiral_window->Button(-text => "Go", -command =>sub {
	$spiral_window->destroy();
	&do_geometry('20','twist',
		     spiral_m   => $m,
		     spiral_rho => $rho);
    })->pack;
}


sub twist_window {
    my $twist_window = MainWindow->new;
    
    my $n = 6;
    
    $twist_window ->title("Simple Twist");
    
    my $frame1 = $twist_window->Frame(-relief => 'groove')->pack(-side => 'left',-expand => 1);
    $frame1->Label(-text => 'Number of sides' )->pack(-side => 'left');
    my $n_entry=$frame1->Entry(-textvariable => \$n,
			       -width => 5,
			       -takefocus => 1)->pack(-side => 'right');
    
    $twist_window->Button(-text => "Go", -command =>sub {
	$twist_window->destroy();
	&do_geometry('99','twist',
		     rot_n   => $n)
	})->pack;
}


sub voderberg_window {
    my $voderberg_window = MainWindow->new;
  
    my $m     = 10;
    my $shift = 1;
    my $depth = 5;

    $voderberg_window ->title("Voderberg spiral parameters window");

    my $frame1 = $voderberg_window->Frame(-relief => 'groove')->pack(-side => 'left',-expand => 1);
    $frame1->Label(-text => 'Axial symmetry (even number)' )->pack(-side => 'left');
    my $m_entry=$frame1->Entry(-textvariable => \$m,
			       -width => 5,
			       -takefocus => 1)->pack(-side => 'right');
  
    my $frame2 = $voderberg_window->Frame(-relief => 'groove')->pack(-side => 'left',-expand => 1);
    $frame2->Label(-text => 'Shift' )->pack(-side => 'left');
    my $shift_entry=$frame2->Entry(-textvariable => \$shift,
				 -width => 5,
				 -takefocus => 1)->pack(-side => 'right');

    my $frame3 = $voderberg_window->Frame(-relief => 'groove')->pack(-side => 'left',-expand => 1);
    $frame3->Label(-text => 'Depth (How large to make spiral!)' )->pack(-side => 'left');
    my $depth_entry=$frame3->Entry(-textvariable => \$depth,
				 -width => 5,
				 -takefocus => 1)->pack(-side => 'right');
   
    $voderberg_window->Button(-text => "Go", -command =>sub {
	$voderberg_window->destroy();
	&do_geometry('21','twist',
		     spiral_m     => $m,
		     spiral_depth => $depth,
		     spiral_shift => $shift);
    })->pack;
}

# Need a window to input tower parameters
sub tower_window {
  my $tower_window = MainWindow->new;

  my $n   = 6;
  my $array = "0.2,0.2,0.2";

  $tower_window ->title("Tower parameters window");

  my $frame1 = $tower_window->Frame(-relief => 'groove')->pack(-side => 'left',-expand => 1);
  $frame1->Label(-text => 'Axial symmetry' )->pack(-side => 'left');
  my $n_entry=$frame1->Entry(-textvariable => \$n,
			     -width => 5,
			     -takefocus => 1)->pack(-side => 'right');
  
  my $frame2 = $tower_window->Frame(-relief => 'groove')->pack(-side => 'left',-expand => 1);
  $frame2->Label(-text => 'Crystal Array' )->pack(-side => 'left');
  my $array_entry=$frame2->Entry(-textvariable => \$array,
			     -width => 20,
			     -takefocus => 1)->pack(-side => 'right');
  
  $tower_window->Button(-text => "Go", -command =>sub {
      $tower_window->destroy();
      &do_geometry('18','twist',
		   rot_n     => $n,
		   tower_array => $array);
  })->pack;
}

sub rosette_window {
  my $rosette_window = MainWindow->new;

  my $n    = 6;
  my $type = 1;

  $rosette_window ->title("Rosette parameters window");

  my $frame1 = $rosette_window->Frame(-relief => 'groove')->pack(-side => 'left',-expand => 1);
  $frame1->Label(-text => 'Rotational symmetry' )->pack(-side => 'left');
  my $n_entry=$frame1->Entry(-textvariable => \$n,
			     -width => 5,
			     -takefocus => 1)->pack(-side => 'right');
  
  $rosette_window->Radiobutton(-text => "Type I",
			       -value => '1',
			       -variable => \$type
			       )->pack(-side => 'left');
  $rosette_window->Radiobutton(-text => "Type II",
			       -value => '2',
			       -variable => \$type
			       )->pack(-side => 'left');


    
  $rosette_window->Button(-text => "Go", -command =>sub {
      $rosette_window->destroy();
      if ($type == 1){
	  &do_geometry('29','twist',
		       rot_n => $n);
      } elsif ($type == 2){
	  &do_geometry('30','twist',
		       rot_n => $n);
      } else {
	  die "Oh no.  Should specifiy either type I or type II!";
      }
  })->pack;
}

sub action_window {
    # User has not selected a tiling yet! 
    if (! $model){
      report("You can only select actions once you have selected a tiling from the geometry menu");
      return 0;

    }

  my $state=$model->state();
  if ($state ne 'tiling'){
      report("You can only select actions before Origamifying your tiling");
      return 0;
  }

  my $mode_window = MainWindow->new;
  $mode_window->title("Action selection");

  $mode_window->Radiobutton(-text => "None",
			    -value => "none",
			    -variable => \$action,
			    -command => sub {&change_mode('none')})->pack(-side => 'left');
  $mode_window->Radiobutton(-text => "Move",
			    -value => "move",
			    -variable => \$action,
			    -command => sub {&change_mode('move')})->pack(-side => 'left');
  $mode_window->Radiobutton(-text => "Delete",
			    -value => "delete",
			    -variable => \$action,
			    -command => sub {&change_mode('delete')})->pack(-side => 'left');
  $mode_window->Radiobutton(-text => "Decompose",
			    -variable => \$action,
			    -value => "decompose",
			    -command => sub {&change_mode('decompose')})->pack(-side => 'left');
  $mode_window->Radiobutton(-text => "Crystal",
			    -variable => \$action,
			    -value => "crystal",
			    -command => sub {&change_mode('crystal')})->pack(-side => 'left');

  # Entry box for crystal parameters
  my $frame1 = $mode_window->Frame(-relief => 'groove', -width => 20)->pack(-side => 'top',
								   -expand => 1,
								   -padx => 10,
								   -pady => 10);

  $crystal_param='0.5';
  $frame1->Label(-text => 'Crystal parameters' )->pack(-side => 'left');
  $frame1->Entry(-textvariable => \$crystal_param,
		 -width => 15,
		 -takefocus => 1)->pack(-side => 'right');

  # Entry box for instar parameters
  my $frame2 = $mode_window->Frame(-relief => 'groove', -width => 20)->pack(-side => 'top',
								   -expand => 1,
								   -padx => 10,
								   -pady => 10);

  $mode_window->Button(-text => "close", -command =>sub {$mode_window->destroy;})->pack;
}

sub report {
  my $report = shift @_;
  my $reportwindow = MainWindow->new;
  $reportwindow->Label(-text=>"$report")->pack;
  $reportwindow->Button(-text => "close", -command =>sub {$reportwindow->destroy;})->pack;
}

sub print_to_file_window {
  my $print_window = MainWindow->new;
  my $file;

  if (! defined $print_window){
      die "In gui.pl print_to_file print_window not defined!";
  }

  my $frame1 = $print_window->Frame(-relief => 'groove', -width => 20)->pack(-side => 'left',
								 -expand => 1,
								 -padx => 10,
								 -pady => 10);

  $frame1->Label(-text => 'Save as file name' )->pack(-side => 'left');
  my $print_entry=$frame1->Entry(-textvariable => \$file,
				 -width => 15,
				 -takefocus => 1)->pack(-side => 'right');
  $print_entry->bind("<Key-Return>",sub {
      $view->handle()->postscript(-file => "$file");
      &report ("Printing canvas to file $file"); 
  });
  
  $print_window->Button(-text => "save",  -command =>sub {
      $view->handle()->postscript(-file => "$file");
      &report ("Printing canvas to file $file"); 
  })->pack;

  $print_window->Button(-text => "close", -command =>sub {$print_window->destroy;})->pack;
}



sub print_to_pdf_window {
  my $print_window = MainWindow->new;
  my $file;

  if (! defined $print_window){
      die "In gui.pl print_to_file print_window not defined!";
  }

  my $frame1 = $print_window->Frame(-relief => 'groove', -width => 20)->pack(-side => 'left',
								 -expand => 1,
								 -padx => 10,
								 -pady => 10);

  $frame1->Label(-text => 'Save as file name' )->pack(-side => 'left');
  my $print_entry=$frame1->Entry(-textvariable => \$file,
				 -width => 15,
				 -takefocus => 1)->pack(-side => 'right');
  $print_entry->bind("<Key-Return>",sub {
      my $view2;
      $view2 = new PDF($model,$file);

      $view2->show($view->show());
      $view2->render();
      &report ("Printing canvas to PDF file $file"); 
  });
  
  $print_window->Button(-text => "save",  -command =>sub {
      my $view2;
      $view2 = new PDF($model,$file);
      $view2->show($view->show());
      $view2->render();
      &report ("Printing canvas to PDF file $file"); 
  })->pack;

  $print_window->Button(-text => "close", -command =>sub {$print_window->destroy;})->pack;


}

sub drag {
    my ($canvas, $x, $y) = @_;
    # Find where button is released
    $mw->configure(-cursor=>'hand2');
    # Bind release
    $canvas->Tk::bind("<ButtonRelease-1>", [\&drop,$x,$y,Ev('x'), Ev('y')]);
}

sub drop {
    my ($canvas, $x1,$y1,$x2,$y2) = @_;
    $mw->configure(-cursor=>'X_cursor');
    my $x_translate=$x2-$x1;
    my $y_translate=$y2-$y1;
    &change_translate($canvas,$x_translate,$y_translate);
}

sub initialise_display {
    $canvas_scale  = 1; # This is the absolute scaling
    $centre_state=0;
    $light_state=0;
    $fill_state=0;
    
    if ($view and $view->handle()){
	&change_mode('move'); # Default mode
    }
}


__END__
